<?php
/**
 * Companies editing controller
 * 
 * Covers all of the functionality related to adding, editing or deleting
 *  
 * @package dotproject3
 * @subpackage Companies
 */
class Companies_EditController extends Zend_Controller_Action
{
	/**
	 * Instantiate form for this object, and populate with default values.
	 */
	public function getForm()
	{
		$form = new Companies_Form_Company();
		

		// Populate Owner SELECT
		$tbl_users = new Admin_Model_DbTable_Users();
		$select = $tbl_users->select()->from($tbl_users, array('user_id', 'user_username'));
		$users = $tbl_users->fetchAll($select)->toArray();
		
		$users_hash = Array();
		foreach ($users as $u) {
			$users_hash[$u['user_id']] = $u['user_username'];
		}
		
		$form->getElement('company_owner')->setMultiOptions($users_hash);
		
		// populate company type
		$type_options = DP_Sysvals::get('CompanyType');
		$form->getElement('company_type')->setMultiOptions($type_options);
		
		// Add JsValidate Decorator
		$form->addElementPrefixPath('DP_Form_Decorator',
                            'DP/Form/Decorator/',
                            'decorator');
		
		// Testing of JsValidate
		//$form->getElement('company_name')->addDecorators(array('JsValidate'));
		
		return $form;
	}

	/**
	 * Create a new object.
	 */
	public function newAction()
	{
		$this->_helper->viewRenderer('object');
		
		$this->view->heading = 'New company';
		$form = $this->getForm();
		$this->view->form = $form;
	}
	
	
	/**
	 * Edit an existing object.
	 */
	public function objectAction()
	{
		$id = $this->getRequest()->getParam('id');
		$form = $this->getForm();

		$tbl = DP_DbTable::factory($this->getRequest()->getModuleName());
		$obj = $tbl->find($id);
			
		if ($obj->count() < 1) { // No matching object found
			throw new DP_Exception_ObjectNotFoundException('Could not find project with the specified id.', 0);
		}
			
		$obj_hash = $obj->current()->toArray(); // Transform the current row to array.
		$form->populate($obj_hash);
			
		$this->view->heading = $obj_hash['company_name'];
		$this->view->form = $form;
	}
	
	/**
	 * Save a project, submitted from objectAction()
	 */
	public function saveAction()
	{
		$this->_helper->viewRenderer('object');
		
		$form = $this->getForm();
		
		if (!$form->isValid($_POST)) {
			$this->view->form = $form;
		} else {
			$this->_helper->viewRenderer->setNoRender();

			$this->_helper->saveObject($form, 'company_id');
			
			$this->_redirector = $this->_helper->getHelper('Redirector');
			$this->_redirector->setGoto('index', 'index', 'companies');
            $this->_redirector->redirectAndExit(); 		
		}
	}
	
}