<?php

/**
 * Form definition for the Company editing form.
 * 
 * @author ebrosnan
 * @package dotproject3
 * @subpackage Companies
 */
class Companies_Form_Company extends Zend_Form
{
    /**
	 * All objects inheriting Zend_Form should implement init() for setting up elements and so forth.
     */
    public function init()
    {
        $this->setName('frmCompany');
        $this->setMethod('POST');
        
        // company_id INT PRIMARY KEY AUTO_INCREMENT
        $this->addElement('hidden', 'company_id');
        
        // company_name	varchar(100)	YES
        $this->addElement('text', 'company_name', Array(
        	'label' => 'Company Name',
        	'required' => true,
        	'attribs' => array(
        		'size' => 50
        	),
        	'validators' => array(
        		array('validator' => 'StringLength', 
        			'options' =>
        				array( 'min' => 1, 'max' => 100 )
        		)
        	)
        ));
        
        // company_email	varchar(255)	YES			
        $this->addElement('text', 'company_email', Array(
        	'label' => 'Company E-mail',
        	'required' => false,
        	'attribs' => array(
        		'size' => 30
        	),
        	'validators' => array(
        		'EmailAddress',
        		array('validator' => 'StringLength', 
        			'options' =>
        				array( 'min' => 1, 'max' => 255 )
        		)
        	)
        ));    

        //company_phone1	varchar(30)	YES			
        $this->addElement('text', 'company_phone1', Array(
        	'label' => 'Phone',
        	'required' => false,
        	'attribs' => array(
        		'size' => 30
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 30 )
        		)
        	)
        ));
        
        //company_phone2	varchar(30)	YES			
        $this->addElement('text', 'company_phone2', Array(
        	'label' => 'Phone 2',
        	'required' => false,
        	'attribs' => array(
        		'size' => 30
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 30 )
        		)
        	)
        ));
        
        //company_fax	varchar(30)	YES			
        $this->addElement('text', 'company_fax', Array(
        	'label' => 'Fax',
        	'required' => false,
        	'attribs' => array(
        		'size' => 30
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 30 )
        		)
        	)
        ));
        
        //company_address1	varchar(50)	YES			
        $this->addElement('text', 'company_address1', Array(
        	'label' => 'Address (Line 1)',
        	'required' => false,
        	'attribs' => array(
        		'size' => 50
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 50 )
        		)
        	)
        ));   

        //company_address2	varchar(50)	YES			
        $this->addElement('text', 'company_address2', Array(
        	'label' => 'Address (Line 2)',
        	'required' => false,
        	'attribs' => array(
        		'size' => 50
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 50 )
        		)
        	)
        ));   

        //company_city	varchar(30)	YES
        $this->addElement('text', 'company_city', Array(
        	'label' => 'City',
        	'required' => false,
        	'attribs' => array(
        		'size' => 20
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 30 )
        		)
        	)
        ));
        
        //company_state	varchar(30)	YES			
        $this->addElement('text', 'company_state', Array(
        	'label' => 'State',
        	'required' => false,
        	'attribs' => array(
        		'size' => 20
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 30 )
        		)
        	)
        ));
        
        //company_zip	varchar(11)	YES			
        $this->addElement('text', 'company_zip', Array(
        	'label' => 'Zip',
        	'required' => false,
        	'attribs' => array(
        		'size' => 11
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 11 )
        		)
        	)
        ));
        
        //company_primary_url	varchar(255)	YES			s
        $this->addElement('text', 'company_primary_url', Array(
        	'label' => 'Zip',
        	'required' => false,
        	'attribs' => array(
        		'size' => 50
        	),
        	'validators' => array(
        		array('validator' => 'StringLength',
        			'options' =>
        				array( 'min' => 1, 'max' => 255 )
        		)
        	)
        ));
        
        //company_description	text	YES			
        $this->addElement('textarea', 'company_description', Array(
        	'label' => 'Company description',
        	'required' => false,
        	'attribs' => array(
        		'rows' => 10
        	)
        ));
        
        //company_owner	int(11)	NO	MUL	0	
        $this->addElement('select', 'company_owner', Array(
        	'label' => 'Company owner',
        	'required' => false
        ));

        //company_type	int(3)	NO		0	
        $this->addElement('select', 'company_type', Array(
        	'label' => 'Company type',
        	'required' => false
        ));
        
       	// BUTTONS	
        $this->addElement('submit', 'Save');
        $this->addElement('button', 'Cancel', Array(
                'attribs' => Array(
                        'onClick' => 'navigateUp();'
                )
        ));

        // Main Group
        $this->addDisplayGroup(
                Array(
                        'company_name',
                        'company_email',
                        'company_phone1',
                        'company_phone2',
                        'company_fax'
                ),
                'main',
                Array(
                        'legend' => 'Main Details',
                        'class' => 'primary'
                )
        );

        // Address Group
        $this->addDisplayGroup(
                Array(
                        'company_address1',
                        'company_address2',
                        'company_city',
                        'company_state',
                        'company_zip'
                ),
                'address',
                Array(
                        'legend' => 'Address'
                )
        );

        // Details Group
        $this->addDisplayGroup(
                Array(
                        'company_url',
                        'company_owner',
                        'company_type',
                        'company_description'
                ),
                'details',
                Array(
                        'legend' => 'Details'
                )
        );

        // Hidden Group
        $this->addDisplayGroup(
                Array(
                    'company_id'
                ),
                'hidden'
        );
                
        // Buttons Group
        $this->addDisplayGroup(
                Array(
                        'Save',
                        'Cancel'
                ),
                'buttons'
        );
    }
}