<?php
/**
 * Class definition for dotproject3 default index controller.
 * @author ebrosnan
 * @package dotproject
 * @subpackage core
 *
 */

/** DP_Auth_Adapter **/
//require_once 'DP/Auth/Adapter.php';

/**
 * Index controller for the default module. Provides an entry point to the application.
 * 
 * Unless specifically configured, all requests are redirected here if the user is not authenticated.
 * The authentication check is performed by DP_Controller_Plugin_LoginCheck
 * 
 * @author ebrosnan
 * @package dotproject
 * @subpackage core
 */
class IndexController extends Zend_Controller_Action
{

    // Here we make sure that the layout is set to login, to avoid displaying navigational components.
    public function init()
    {
       $this->getHelper('layout')->setLayout('login');
    }

    /**
     * Log out of the system. Destroying all session variables.
     * 
     */
    public function logoutAction()
    {
    	Zend_Auth::getInstance()->clearIdentity();
    	$this->_helper->redirector('index','index','home');
    }
    
    /**
     * This action is the entry point for the application in general. It presents a login page which will
     * be re-submitted to itself. On success, the user is forwarded to the home module.
     * 
     */
    public function indexAction()
    {	
		$auth = Zend_Auth::getInstance();
        $login_form = new DP_Form_Login();

        if ($this->getRequest()->getParam('login') != null ) {
	        if ($login_form->isValid($_POST)) {
	            
	            $username = $login_form->getValue('username');
	            $password = $login_form->getValue('password');
	            
	            
	            // TODO - This should be configurable. At the moment it will be a static DbTable Auth Adapter.
	            $adapter = DP_Auth_Adapter::factory('DbTable', new Zend_Config(array(
	                                                                                'username' => $username,
	                                                                                'password' => md5($password)
	            )));

	            $result = $auth->authenticate($adapter);
	            
	            if ($result->isValid()) {
    				$storage = $auth->getStorage();
    				
    				// Omit the password from the result stored in session because if session vars can be dumped then
    				// User can reverse the password.
    				$storage->write($adapter->getResultRowObject(array(
        				'user_id',
    					'user_contact',
    					'user_username',
    					'user_parent',
    					'user_type',
    					'user_company',
    					'user_department',
    					'user_owner',
    					'user_signature'
    				)));
	            	
	                $this->_helper->redirector('index','index','home');
	            } else {
	            	
	            	// Do not display any specifics as it could be considered a security vulnerability.
	            	switch ($result->getCode()) {

	            		case Zend_Auth_Result::SUCCESS:
	            		case Zend_Auth_Result::FAILURE:
	            		case Zend_Auth_Result::FAILURE_IDENTITY_NOT_FOUND:
	            		case Zend_Auth_Result::FAILURE_IDENTITY_AMBIGUOUS:
	            		case Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID:
	            		case Zend_Auth_Result::FAILURE_UNCATEGORIZED:
	            		default:
	            			$this->view->messages = $result->getMessages();
	            	}
	            }
	        }
        } else if ($auth->hasIdentity()) {
		// Already logged in, just push them to the home page.
			$this->_helper->redirector('index','index','home');
		} else {
			$this->view->form = $login_form;
		}
    }
}
