<?php
/**
 * Projects Editing Controller
 * 
 * Covers all of the functionality related to adding, editing or deleting projects.
 * 
 * @author ebrosnan
 * @package dotproject3
 * @subpackage Projects
 */
class Projects_EditController extends Zend_Controller_Action
{
	/**
	 * Instantiate form for this object, and populate with default values.
	 */
	public function getForm()
	{
		$form = new Projects_Form_Project();
		
		// Pre-populated Data.
		$status_options = DP_Sysvals::get('ProjectStatus');
		$form->getElement('project_status')->setMultiOptions($status_options);
		
		$type_options = DP_Sysvals::get('ProjectType');
		$form->getElement('project_type')->setMultiOptions($type_options);
		
		$priority_options = DP_Sysvals::get('ProjectPriority');
		$form->getElement('project_priority')->setMultiOptions($priority_options);

		
		// Populate Companies SELECT
		$tbl_companies = new Companies_Model_DbTable_Companies();

		// TODO: There must be an easier way to build a hash from a 2 column result set?
		$select = $tbl_companies->select()->from($tbl_companies, array('company_id', 'company_name'));
		$companies = $tbl_companies->fetchAll($select)->toArray();
		
		$companies_hash = Array();
		foreach ($companies as $c) {
			$companies_hash[$c['company_id']] = $c['company_name'];
		}
		
		$form->getElement('project_company')->setMultiOptions($companies_hash);
	
		
		// Populate Owner SELECT
		$tbl_users = new Admin_Model_DbTable_Users();
		$select = $tbl_users->select()->from($tbl_users, array('user_id', 'user_username'));
		$users = $tbl_users->fetchAll($select)->toArray();
		
		$users_hash = Array();
		foreach ($users as $u) {
			$users_hash[$u['user_id']] = $u['user_username'];
		}
		
		$form->getElement('project_owner')->setMultiOptions($users_hash);
		
		return $form;
	}
	
	/**
	 * Create a new object.
	 */
	public function newAction()
	{
		$this->_helper->viewRenderer('object');
		
		$this->view->heading = 'New project';
		$form = $this->getForm();
		$this->view->form = $form;
	}
	
	/**
	 * Edit an existing object.
	 */
	public function objectAction()
	{
		$id = $this->getRequest()->getParam('id');
		$form = $this->getForm();

		$tbl = DP_DbTable::factory($this->getRequest()->getModuleName());
		$obj = $tbl->find($id);
			
		if ($obj->count() < 1) { // No matching object found
			throw new DP_Exception_ObjectNotFoundException('Could not find project with the specified id.', 0);
		}
			
		$project = $obj->current()->toArray(); // Transform the current row to array.
		$form->populate($project);
			
		$this->view->heading = $project['project_name'];
		$this->view->form = $form;
	}
	
	/**
	 * Save a project, submitted from objectAction()
	 */
	public function saveAction()
	{
		$this->_helper->viewRenderer('object');
		
		$form = $this->getForm();
		
		if (!$form->isValid($_POST)) {
			$this->view->form = $form;
		} else {
			$this->_helper->viewRenderer->setNoRender();

			$this->_helper->saveObject($form, 'project_id');
			
			$this->_redirector = $this->_helper->getHelper('Redirector');
			$this->_redirector->setGoto('index', 'index', 'projects');
            $this->_redirector->redirectAndExit(); 		
		}
	}
	
}