<?php

/**
 * Form definition for the Project editing form.
 * 
 * @author ebrosnan
 * @package dotproject3
 * @subpackage Projects
 */
class Projects_Form_Project extends Zend_Form
{
    /**
	 * All objects inheriting Zend_Form should implement init() for setting up elements and so forth.
     */
    public function init()
    {
        $this->setName('frmProject');
        $this->setMethod('POST');
        
        // 	project_id
        $this->addElement('hidden', 'project_id');
        
        //	project_name VARCHAR(255),
        $this->addElement('text', 'project_name', Array(
        	'label' => 'Project Name',
        	'required' => true,
        	'attribs' => array(
        		'size' => 25,
        		'onBlur' => 'setShortName()'
        	),
        	'validators' => array(
        		array('validator' => 'StringLength', 
        			'options' =>
        				array( 'min' => 1, 'max' => 255 )
        		)
        	)
        ));
        
        //	project_owner INT DEFAULT 0,
        $this->addElement('select', 'project_owner', Array(
        	'label' => 'Project Owner'
        ));
        
        //	project_company INT DEFAULT 0 NOT NULL,
        $this->addElement('select', 'project_company', Array(
        	'label' => 'Company',
        	'required' => true,
        	'attribs' => Array(
        		'id' => 'project-company-select'
        	)
        ));
        
        //	project_start_date DATETIME,
        $this->addElement('text', 'project_start_date', Array(
                'label' => 'Start Date',
                'required' => false,
                'attribs' => Array(
                        'size' => 25
                ),
                'validators' => Array(
                        array('validator' => 'Date')
                )
        ));

        //	project_end_date DATETIME,
        $this->addElement('text', 'project_end_date', Array(
                'label' => 'Target Finish Date',
                'required' => false,
                'attribs' => Array(
                        'size' => 25
                ),
                'validators' => Array(
                        array('validator' => 'Date')
                )
        ));		

        //	project_target_budget DECIMAL(10 , 2) DEFAULT 0.00,
        $this->addElement('text', 'project_target_budget', Array(
                'label' => 'Target Budget',
                'required' => false,
                'validators' => Array(
                        //array('validator' => 'Regex', 'options' => Array(
                        //	'pattern' => '[0-9]+\.[0-9]+'
                        //))
                )
        ));

        //	project_actual_end_date DATETIME,
        $this->addElement('text', 'project_actual_end_date', Array(
                'label' => 'Actual End Date',
                'required' => false,
                'attribs' => Array(
                        'size' => 25
                ),
                'validators' => Array(
                        array('validator' => 'Date')
                )
        ));

        //	project_actual_budget DECIMAL(10 , 2) DEFAULT 0.00,
        $this->addElement('text', 'project_actual_budget', Array(
                'label' => 'Actual Budget',
                'required' => false,
                'validators' => Array(
                        //array('validator' => 'Regex', 'options' => Array(
                        //	'pattern' => '^[0-9]*\.[0-9]{2}$'
                        //))
                )
        ));		

        //	project_url VARCHAR(255),
        $this->addElement('text', 'project_url', Array(
                'label' => 'URL',
                'required' => false,
                'attribs' => Array(
                        'size' => 60
                ),
                'validators' => Array(
                        array('validator' => 'StringLength', 
                        'options' =>
                                array( 'min' => 1, 'max' => 255 )
                )
                )
        ));

        //	project_demo_url VARCHAR(255),
        $this->addElement('text', 'project_demo_url', Array(
                'label' => 'Staging URL',
                'required' => false,
                'attribs' => Array(
                        'size' => 60
                ),
                'validators' => Array(
                        array('validator' => 'StringLength', 
                        'options' =>
                                array( 'min' => 1, 'max' => 255 )
                )
                )
        ));

        //	project_priority TINYINT DEFAULT 0,
        $this->addElement('select', 'project_priority', Array(
                'label' => 'Priority',
                'required' => false
        ));

        //	project_short_name VARCHAR(10),
        $this->addElement('text', 'project_short_name', Array(
                'label' => 'Short Name',
                'required' => false,
                'attribs' => Array(
                        'size' => 10
                ),
                'validators' => Array(
                        array('validator' => 'StringLength', 
                        'options' =>
                                array( 'min' => 1, 'max' => 10 )
                )
                )			
        ));

        //	project_color_identifier VARCHAR(6) DEFAULT eeeeee,
        $this->addElement('text', 'project_color_identifier', Array(
                'label' => 'Color Identifier',
                'required' => false,
                'attribs' => Array(
                        'size' => 6
                ),
                'validators' => Array(
                        array('validator' => 'StringLength', 
                        'options' =>
                                array( 'min' => 1, 'max' => 6 )
                )
                )			
        ));

        //	project_type SMALLINT DEFAULT 0 NOT NULL,
        $this->addElement('select', 'project_type', Array(
                'label' => 'Type',
                'required' => false
        ));

        //	project_status INT DEFAULT 0,
        $this->addElement('select', 'project_status', Array(
                'label' => 'Status',
                'required' => false
        ));

        //	project_percent_complete TINYINT DEFAULT 0,
        $this->addElement('hidden', 'project_percent_complete', Array(
                'label' => 'Percent Complete'
        ));

        //	project_department INT DEFAULT 0 NOT NULL,
        $this->addElement('select', 'project_department', Array(
                'label' => 'Department'
        ));

        //	project_creator INT DEFAULT 0,
        $this->addElement('hidden', 'project_creator', Array(
                'label' => null //'Creator'
        ));

        //	project_description TEXT,
        $this->addElement('textarea', 'project_description', Array(
                'label' => 'Description',
                'required' => false,
                'attribs' => Array(
                        'rows' => 10,
                        'cols' => 40
                )
        ));

        //	project_private TINYINT UNSIGNED DEFAULT 0,
        $this->addElement('checkbox', 'project_private', Array(
                'label' => 'Private Entry'
        ));


        // BUTTONS

        $this->addElement('submit', 'Save');
        $this->addElement('button', 'Cancel', Array(
                'attribs' => Array(
                        'onClick' => 'navigateUp();'
                )
        ));

        // Main Group	
        $this->addDisplayGroup(
                Array(
                        'project_name',
                        'project_short_name',
                        'project_company',
                        'project_priority',
                        'project_color_identifier'
                ),
                'main',
                Array(
                        'legend' => 'Main Details',
                        'class' => 'primary'
                )
        );

        // Time and Budget Group
        $this->addDisplayGroup(
                Array(
                        'project_start_date',
                        'project_end_date',
                        'project_target_budget',
                        'project_actual_end_date',
                        'project_actual_budget'
                ),
                'timeandbudget',
                Array(
                        'legend' => 'Time/Budget Constraints',
                        'class' => 'secondary'
                )
        );		

        // Details Group
        $this->addDisplayGroup(
                Array(
                        'project_type',
                        'project_department',
                        'project_owner',
                        'project_private',
                        'project_url',
                        'project_demo_url',
                        'project_description'
                ),
                'details',
                Array(
                        'legend' => 'Details',
                        'class' => 'secondary'
                )
        );

        // Status Group
        $this->addDisplayGroup(
                Array(
                        'project_status',
                        'project_percent_complete'
                ),
                'status',
                Array(
                        'legend' => 'Status',
                        'class' => 'secondary'
                )
        );

        // Buttons Group and Hidden elements.
        $this->addDisplayGroup(
                Array(
                        'Save',
                        'Cancel',
                        'project_id',
                        'project_creator'
                ),
                'buttons'
        );


    }
}