<?php
/**
 * This is the file containing the definition of the login controller
 * @author ajdonnison
 * @version 3.0
 * @package dotproject
 */

/**
 * The LoginController handles the login page and its acceptance
 * @package dotproject
 * @subpackage default
 */
class LoginController extends Zend_Controller_Action
{
	/**
	 * Displays the login page.
	 * 
	 * Stores the URL requested before the login page was displayed, so that the user can be
	 * redirected to their originally requested location after they log in.
	 */
	public function indexAction()
	{		
		$layout = $this->_helper->layout();
		//$layout->dialog = 1;
		$layout->setLayout('layout-login');
		
		$AppUI = DP_AppUI::getInstance();
		$this->view->version = $AppUI->getVersion();

		$this->view->config = DP_Config::getConfig();
		$this->view->message = implode('<br />',$this->getHelper('FlashMessenger')->getMessages());
		
		$this->view->random_salt = 'asffRDsdgWTEsgdzvxsdfGBd'; // @TODO remove temp static see.
	}	


	public function xhrAuthenticate()
	{
		$this->_helper->layout->disableLayout();
		$writer = new Zend_Log_Writer_Firebug();
		$logger = new Zend_Log($writer);
		
		
		$request = $this->getRequest();
		
		$db = DP_Config::getDB();
		
		$username = $request->username;
		$password_hashed = $request->password;
		
		// hash with seed => MD5 hashed password
		// compare MD5 with db
		$authAdapter = DP_Auth::factory();
		$authAdapter->setIdentity($username);
		$authAdapter->setCredential($password_hashed);
		
		$auth = Zend_Auth::getInstance();
		$result = $auth->authenticate($authAdapter);
		
		if ($result->isValid()) {
			// Associate user data with session.
			// Associate resolved user permissions and group memberships with session.
			$person = DP_Person::factory($auth->getIdentity());
			
			$user_session = Array(
					'person'=>$person,
					'undo'=>null // Store undo memento
			);

			$auth->getStorage()->write($user_session);
			
			// Provide result via JSON.
		} else {
			// Provide result via JSON.
		}
		
	}

	/**
	 * Save and process the submitted login details.
	 * 
	 * If the user authenticates successfully then redirect to their originally requested URL,
	 * otherwise display 'failed' message.
	 */
	public function saveAction()
	{
		$request = $this->getRequest();
		$form = $this->getLoginForm();
		
		if($form->isValid($_POST)) {
			$db = DP_Config::getDB();
			
			$credentials = $form->getValues();
			
			$authAdapter = DP_Auth::factory();
			$authAdapter->setIdentity($credentials['username']);
			$authAdapter->setCredential($credentials['password']);
	
			$auth = Zend_Auth::getInstance();
			$result = $auth->authenticate($authAdapter);
			
			if ($result->isValid()) {
				$person = DP_Person::factory($auth->getIdentity());
				// $undo_manager = DP_UndoManager::factory()
				$user_session = Array(
						'person'=>$person,
						'undo'=>null
				);

				$auth->getStorage()->write($user_session);
				
				// write user table data into session
				/*
				$user_session = new DP_User_Session($auth->getIdentity());
				$user_session->load();
				*/
				
				$redirect = urldecode($this->getRequest()->getParam('redirect'));
				$this->getResponse()->setRedirect($redirect);
				$this->getResponse()->sendHeaders();
				//error_log('Redirecting to '.$redirect);
				exit;
			} else {
				$this->_helper->FlashMessenger('Login Failed.');
				$this->_forward('index');
			}
		}		
	}
}
?>
