<?php
/**
 * Class definition for DP_Auth_Adapter
 * 
 * @author ebrosnan
 */

/**
 * Zend_Auth_Adapter Factory Class
 * 
 * Provides a factory method for instantiating Zend_Auth_Adapters based on application config.
 * Allows us to insert our own adapter configuration, read from our own configuration source.
 * 
 * Normally ZF does not provide a factory class with pre-configuration.
 * 
 * @package dotproject3
 * @subpackage DP_Auth
 * @example $my_zend_auth_adapter = DP_Auth_Adapter::factory('dbtable', $options)
 * @author ebrosnan
 * @see Zend_Auth
 * @see Zend_Auth_Adapter_Interface
 */
class DP_Auth_Adapter
{
    /**
     * Factory method for instantiating classes which implement Zend_Auth_Adapter_Interface
     * 
     * Provides a per-config method of instantiating authentication adapters instead of branching the login
     * code of an application.
     * 
     * @param String $adapterName one of ('dbtable', 'dummy')
     * @param Zend_Config $adapterOptions Valid options passed to Zend_Auth_Adapter
     * @return Zend_Auth_Adapter_Interface Authentication adapter
     */
    public static function factory($adapterName, Zend_Config $adapterOptions)
    {
        $adapterNameLower = strtolower($adapterName);
        
        switch($adapterNameLower) {
            case 'dbtable':
            	$username_column = 'user_username';
            	$password_column = 'user_password';
            	
                // return a dbtable adapter.
                $adapter = new Zend_Auth_Adapter_DbTable(Zend_Db_Table::getDefaultAdapter(),
                                                        'users',
                                                        $username_column,
                                                        $password_column
                );
                
                
                if ($adapterOptions->username) {
                    $adapter->setIdentity($adapterOptions->username);
                }
                
                if ($adapterOptions->password) {
	                $adapter->setCredential($adapterOptions->password);
                }

                return $adapter;
                
                break;
            case 'ldap':
                // return an ldap adapter, and so on.
                break;
            case 'dummy':
                return new DP_Auth_Adapter_Dummy('dummy');
                break;
        }
    }
    
}