<?php
/**
 * Class definition for the LoginCheck plugin.
 * 
 * @package dotproject
 * @subpackage DP
 */

/**
 * @package dotproject3
 * @subpackage DP
 * 
 * The LoginCheck class makes sure before any dispatch that the user is logged in.
 * Modules which are publicly viewable will not need this plugin.
 *
 */
class DP_Controller_Plugin_LoginCheck extends Zend_Controller_Plugin_Abstract
{
    /**
     * Hook for Zend_Controller_Front's routeShutdown. Check to see if a user is logged in. If not, then redirect.
     * 
	 * routeShutdown() is called before the dispatch loop. This seems to be less messy than
	 * using preDispatch() which forces the loop to iterate again when we change the request parameters.
	 * 
	 * @see Zend_Controller_Plugin_Abstract::routeShutdown()
     */
    public function routeShutdown(Zend_Controller_Request_Abstract $request)
    {
        $auth = Zend_Auth::getInstance();
        
        if (!$auth->hasIdentity()) {
        	$original_request = clone $request;
        	$request->setParam('redirect', $original_request);
        	
            $request->setModuleName('default');
            $request->setControllerName('index');
            $request->setActionName('index');
        }
    }
}