<?php
/**
 * Sysvals model class.
 * 
 * Interacts with the database to provide sysval related functions, mostly lookup.
 * 
 * Uses a static class variable to maintain a cached copy of the sysvals results.
 */
class DP_Sysvals
{
	/**
	 * Cached copy of sysvals retrieved from sysvals table.
	 * 
	 * @var Array [Title] => [Array of key=>value]
	 */
	protected static $_sysvals;
	
	/**
	 * Fetch a sysval or collection of sysvals.
	 * 
	 * You must supply a title (a type of sysval to retrieve). Optionally you can supply the key.
	 * Without the key this method will return all of the sysvals of the specified type.
	 * 
	 * @param String $title Name of the type of sysval to retrieve eg. ProjectStatus, CompanyType...
	 * @param String $key The key to look up, of the type specified in $title. If null, returns all valid values.
	 */
	public static function get( $title, $key = null )
	{
		if (!isset(self::$_sysvals[$title]) || !is_array(self::$_sysvals)) {
		
			$tbl = new DP_DbTable_Sysvals();
			$select = $tbl->select()->where('sysval_title = ?', $title);
			
			$rs = $tbl->fetchAll($select);
	
			$sysvals = array();
			// TODO: probably a specific function for building this array in-place.
			// Note:  This handles both the old-style sysvals and the newer style - we probably need to decide which is going to be the right one going forward and drop support for the other.
			foreach ($rs as $r) {
				$r_val = $r['sysval_value'];
				if (isset($r['sysval_value_id'])) {
					$sysvals[$r['sysval_value_id']] = $r_val;
				} else {
					$sv_pairs = explode("\n", $r_val);
					foreach ($sv_pairs as $v) {
						$parts = explode("|", $v, 2);
						$sysvals[$parts[0]] = trim($parts[1]); // Some vals contain \r characters.
					}
				}
			}
			
			self::$_sysvals[$title] = $sysvals;
		}
		
		if ($key !== null) {
			return self::$_sysvals[$title][$key];
		} else {
			return self::$_sysvals[$title];
		}
	}
}

