<?php
/**
 * A server side representation of the state of a list widget.
 * 
 * This is intended to replace the old codebase DP_List_Dynamic as the primary object to use for the list
 * state.
 * 
 * The purpose of the stateful list view is to:
 * - Capture POST and GET variables relevant to the list or list child views.
 * - Propagate those variables to child views.
 * - Hold state information about the list: column definitions, maximum number of rows.
 * 
 * @author ebrosnan
 * @package dotproject3
 * @subpackage DP_View
 */
class DP_View_Stateful_List extends DP_View_Stateful
{
	/**
	 * The view helper to use when rendering this object.
	 * @var Zend_View_Helper Helper
	 */
	public $helper = 'dpViewList';
	
	/**
	 * @var DP_List The DP_List instance to use for the data source.
	 */
	protected $data_source;
	
	/**
	 * @var DP_View_Stateful_Columns instance of a columns object.
	 */
	protected $columns;
	
	/**
	 * DP_View_Stateful_List Constructor
	 * @param String $id Identifier.
	 */
	public function __construct($id)
	{
		parent::__construct($id);
	}
	
	/**
	 * Setter to set the list views columns.
	 * 
	 * @param DP_View_Stateful_Columns $cols
	 * @return null
	 */
	public function setColumns(DP_View_Stateful_Columns $cols)
	{
		$this->columns = $cols;
	}
	
	/**
	 * Getter which returns the instance of the columns object.
	 * 
	 * @return DP_View_Stateful_Columns Instance of the columns.
	 */
	public function getColumns()
	{
		return $this->columns;
	}
	
	/**
	 * Set the data source to use for this list view.
	 * 
	 * Changes in the list view state may affect the data source which is why this list view maintains a reference to the instance.
	 * The view also receives the hook where the view is about to render. The datasource needs notification because the query should be run
	 * just before the view goes to render.
	 * 
	 * @param DP_List $data_source
	 */
	public function setDataSource(DP_List $data_source)
	{
		$this->data_source = $data_source;	
	}
	
	/**
	 * Get the internally referenced data source.
	 * 
	 * @return DP_List List data source.
	 */
	public function getDataSource()
	{
		return $this->data_source;
	}
}