<?php

/**
 * A text field which will search a specific field.
 * 
 * The search filter is rendered as a text field in its own form. It maintains an
 * internal modifier with a specific field set.
 * 
 * @package dotproject3
 * @subpackage DP_View
 */
class DP_View_Stateful_SearchFilter extends DP_View_Stateful
{
	/**
	 * The view helper to use when rendering this object.
	 * @var Zend_View_Helper Helper
	 */
	public $helper = 'dpViewSearchFilter';	
	
	/**
	 * @var $modifier DP_Modifier internal modifier instance.
	 */
	protected $modifier = null;
	
	/**
	 * Constructor
	 */
	public function __construct($id = -1, DP_Modifier $mod)
	{
		parent::__construct($id);
		
		$this->modifier = $mod;
		
		$saved_index_memento = $this->loadState();
		if (null !== $saved_index_memento) {
			$this->setValue($saved_index_memento->getState());
		}
	}
	
	/**
	 * Get the internal instance of the modifier.
	 * 
	 * @return DP_Modifier Instance of a modifier.
	 */
	public function getModifier()
	{
		return $this->modifier;
	}
	
	/**
	 * Set the internally stored value to match.
	 * 
	 * @param String $v
	 */
	public function setValue( $v )
	{
		$this->modifier->setValue($v);
	}
	
	/**
	 * Get the internally stored input value.
	 * 
	 * @return String Value being searched against.
	 */
	public function getValue()
	{
		return 'test';
	}
	
	// From DP_View_Stateful
	
	public function updateStateFromServer($request)
	{
		parent::updateStateFromServer($request);
		
		// Receive state change.
		if ($request->getParam('view_id') == $this->id() && $request->getParam($this->id().'-input') !== null) {
			
			//Zend_Debug::dump($this->id().' got update value: '.$request->getParam('select'));
			$value = $request->getParam($this->id().'-input');
			
			$this->setValue($value);
		}
		
		// Save state
		if (true === $this->persist_state) {
			$this->saveState(new DP_Memento($this->getValue()));
		}
	}
}