YUI({ lang: 'en-AU' }).use('node', 'event', 'io', 'datasource', 'dataschema', 'dp-datatype', 'dp-search', 'dp-table', 'tabview', function(Y) {

   var tv = new Y.TabView({srcNode: '#company-tabview'});
   tv.render();

    /**
     * Render a project name cell, with link to project.
     * 
     * @param data {Object} Row data.
     * @param field {String} Field name.
     * @param cellNode {Node} Reference to cell node.
     */
    function renderProjectCell( data, field, cellNode ) {
            if (cellNode == null) { throw('cellNode not supplied to formatter.'); }

            var uri = '/projects/edit/object/id/' + data["id"];
            var aNode = Y.Node.create(Y.substitute('<a href="{uri}">{title}</a>', {
                    uri : uri,
                    title : data[field]
            }));
            cellNode.append(aNode);
    }


    var myDataSource = new Y.DataSource.IO({source:"/projects/json/index/company_id/1" });
    myDataSource.plug(Y.Plugin.DataSourceJSONSchema, {
    schema: {
            metaFields : { count: "resultCount" },
            resultListLocator : "results",
            resultFields : [
                { key:"finish", parser:Y.DP.DataType.DateTime.parse }, 
                { key:"start", parser:Y.DP.DataType.DateTime.parse },
                "progress",
                "name",
                "id"
            ]
        }
    });

    // 12 cache entries covers all permutations of sorting (single column sort only)
    myDataSource.plug({fn:Y.Plugin.DataSourceCache, cfg:{max:12}});

    var tbl = new Y.DP.TableBody({ 
            srcNode : '.yui3-dp-table',
            boundingBox : '.yui3-dp-table-body',
            dataSource : myDataSource,
            cells : [
                     { field: "name", renderer: renderProjectCell, width: '150em' },
                     { field: "start", renderer: Y.DP.CellRenderers.niceDate, width: '15em' },
                     { field: "finish", renderer: Y.DP.CellRenderers.niceDate, width: '15em' },
                     { field: "progress", renderer: Y.DP.CellRenderers.progress, width: '20em' }
            ]
    });

    var thead = new Y.DP.TableHeaders({ srcNode: '.yui3-dp-table', boundingBox: '.yui3-dp-table-headers' });
    thead.on('queryUpdate', tbl.handleParameterChange, tbl);
    thead.render();

    tbl.render();
});