YUI({ lang: 'en-AU' }).use('node', 'event', 'io', 'datasource', 'dataschema', 'dp-datatype', 'dp-search', 'dp-table', function(Y) {
	
	/**
	 * Render a project name cell, with link to project.
	 * 
	 * @param data {Object} Row data.
	 * @param field {String} Field name.
	 * @param cellNode {Node} Reference to cell node.
	 */
	function renderCompanyCell( data, field, cellNode ) {
                if (cellNode == null) { 
                    throw('cellNode not supplied to formatter.'); 
                } else {

                    var uri = '/companies/edit/object/id/' + data["id"];
                    var aNode = Y.Node.create(Y.substitute('<a href="{uri}">{title}</a>', {
                            uri : uri,
                            title : data[field]
                    }));
                    cellNode.append(aNode);
                }
	}
	

		
        var myDataSource = new Y.DataSource.IO({source:"/companies/json/index"});

        myDataSource.plug(Y.Plugin.DataSourceJSONSchema, {
            schema: {
                metaFields : { count: "resultCount" },
                resultListLocator : "results",
                resultFields : [
                    "type",
                    "name",
                    "id"
                ]
            }
        });

        // 12 cache entries covers all permutations of sorting (single column sort only)
        myDataSource.plug({fn:Y.Plugin.DataSourceCache, cfg:{max:12}});

        var tbl = new Y.DP.TableBody({ 
                srcNode : '.yui3-dp-table',
                boundingBox : '.yui3-dp-table-body',
                dataSource : myDataSource,
                cells : [
                         { field: "name", renderer: renderCompanyCell, width: '150em' },
                         { field: "type", renderer: Y.DP.CellRenderers.hash('#company-type-options'), width: '15em' },
                ]
        });

        var thead = new Y.DP.TableHeaders({ srcNode: '.yui3-dp-table', boundingBox: '.yui3-dp-table-headers' });
        thead.on('queryUpdate', tbl.handleParameterChange, tbl);
        thead.render();

        var search = new Y.DP.Search({ srcNode: '.yui3-dp-search' });
        search.on('queryUpdate', tbl.handleParameterChange, tbl);
        search.render();

        tbl.render();

});
