YUI({ lang: 'en-AU' }).use('node', 'event', 'io', 'datasource', 'dataschema', 'dp-datatype', 'dp-search', 'dp-table', 'dp-table-selection-plugin', function(Y) {
	
	/**
	 * Render a project name cell, with link to project.
	 * 
	 * @param data {Object} Row data.
	 * @param field {String} Field name.
	 * @param cellNode {Node} Reference to cell node.
	 */
	function renderProjectCell( data, field, cellNode ) {
                if (cellNode == null) { 
                    throw('cellNode not supplied to formatter.'); 
                } else {

                    var uri = '/projects/edit/object/id/' + data["id"];
                    var aNode = Y.Node.create(Y.substitute('<a href="{uri}">{title}</a>', {
                            uri : uri,
                            title : data[field]
                    }));
                    cellNode.append(aNode);
                }
	}
	

		
        var myDataSource = new Y.DataSource.IO({source:"/projects/json/index"});

        myDataSource.plug(Y.Plugin.DataSourceJSONSchema, {
            schema: {
                metaFields : { count: "resultCount" },
                resultListLocator : "results",
                resultFields : [
                "status", 
               // TODO - DataType.Date.parse does not support MySQL DATETIME out of the box. extend with this functionality
                { key:"finish", parser:Y.DP.DataType.DateTime.parse }, 
                { key:"start", parser:Y.DP.DataType.DateTime.parse },
                "progress",
                "name",
                "id"
                ]
            }
        });

        // 12 cache entries covers all permutations of sorting (single column sort only)
        myDataSource.plug({fn:Y.Plugin.DataSourceCache, cfg:{max:12}});
        Y.log(Y.DP.CellRenderers);

        var tbl = new Y.DP.TableBody({ 
                srcNode : '.yui3-dp-table',
                boundingBox : '.yui3-dp-table-body',
                dataSource : myDataSource,
                cells : [
                         { field: "name", renderer: renderProjectCell, width: '150em' },
                         { field: "start" , renderer: Y.DP.CellRenderers.niceDate, width: '15em' },
                         { field: "finish", renderer: Y.DP.CellRenderers.niceDate, width: '15em' },
                         { field: "status", renderer: Y.DP.CellRenderers.hash('#project-status-options'), width: '20em' },
                         { field: "progress", renderer: Y.DP.CellRenderers.progress(), width: '20em' }
                ],
                plugins : [{fn:Y.DP.SelectionPlugin, cfg:{}}]
        });

        var thead = new Y.DP.TableHeaders({ srcNode: '.yui3-dp-table', boundingBox: '.yui3-dp-table-headers' });
        thead.on('queryUpdate', tbl.handleParameterChange, tbl);
        thead.render();

        var search = new Y.DP.Search({ srcNode: '.yui3-dp-search' });
        search.on('queryUpdate', tbl.handleParameterChange, tbl);
        search.render();

        tbl.render();

});
